@@ -9,6 +9,14 @@ from PyQt4 import QtGui
 
 from cola import utils
 from cola import resources
+from cola import i18n
+from cola.decorators import memoize
+
+
+@memoize
+def instance(argv):
+   return QtGui.QApplication(list(argv))
+
 
 class ColaApplication(object):
     """The main cola application
@@ -19,9 +27,13 @@ class ColaApplication(object):
     def __init__(self, argv, locale=None, gui=True):
         """Initialize our QApplication for translation
         """
-        # monkey-patch Qt's app translate() to handle .po files
+        if locale:
+            os.environ['LANG'] = locale
+        i18n.install()
+
+        # monkey-patch Qt's translate() to use our translate()
         if gui:
-            self._app = QtGui.QApplication(argv)
+            self._app = instance(tuple(argv))
             self._app.setWindowIcon(QtGui.QIcon(resources.icon('git.svg')))
             self._translate_base = QtGui.QApplication.translate
             QtGui.QApplication.translate = self.translate
@@ -30,29 +42,26 @@ class ColaApplication(object):
             self._translate_base = QtCore.QCoreApplication.translate
             QtCore.QCoreApplication.translate = self.translate
 
-        # Find the current language settings and apply them
-        if not locale:
-            locale = str(QtCore.QLocale().system().name())
-
-        qmfile = utils.get_qm_for_locale(locale)
-        if os.path.exists(qmfile):
-            translator = QtCore.QTranslator(self._app)
-            translator.load(qmfile)
-            self._app.installTranslator(translator)
-
-    def translate(self, context, *args):
-        """Supports @@noun/@@verb and context-less translation
+    def translate(self, context, txt):
         """
-        # We set the context to '' to properly handle .qm files
-        trtxt = unicode(self._translate_base('', *args))
+        Translate strings with gettext
+
+        Supports @@noun/@@verb specifiers.
+
+        """
+        trtxt = i18n.gettext(txt)
         if trtxt[-6:-4] == '@@': # handle @@verb / @@noun
             trtxt = trtxt[:-6]
         return trtxt
 
     def activeWindow(self):
-        """Wrapper around our QApp.activeWindow()"""
+        """Wrap activeWindow()"""
         return self._app.activeWindow()
 
     def exec_(self):
-        """Wrapper around our QApp.exec_()"""
+        """Wrap exec_()"""
         return self._app.exec_()
+
+    def setStyleSheet(self, txt):
+        """Wrap setStyleSheet(txt)"""
+        return self._app.setStyleSheet(txt)
