@@ -8,17 +8,31 @@ import re
 import os
 import sys
 import errno
+import commands
 import subprocess
+import threading
 
+import cola
 from cola import core
 from cola import errors
+from cola import signals
+from cola.decorators import memoize
+
+cmdlock = threading.Lock()
+
+
+@memoize
+def instance():
+    """Return the GitCola singleton"""
+    return GitCola()
 
 
 def dashify(string):
     return string.replace('_', '-')
 
 # Enables debugging of GitPython's git commands
-GIT_PYTHON_TRACE = os.environ.get("GIT_PYTHON_TRACE", False)
+GIT_PYTHON_TRACE = os.getenv('GIT_PYTHON_TRACE', False)
+GIT_COLA_TRACE = False
 
 execute_kwargs = ('cwd',
                   'istream',
@@ -31,6 +45,7 @@ extra = {}
 if sys.platform == 'win32':
     extra = {'shell': True}
 
+
 class Git(object):
     """
     The Git class manages communication with the Git binary
@@ -62,7 +77,7 @@ class Git(object):
             The command argument list to execute
 
         ``istream``
-            Standard input filehandle passed to subprocess.Popen.
+            Readable filehandle passed to subprocess.Popen.
 
         ``cwd``
             The working directory when running commands.
@@ -84,6 +99,7 @@ class Git(object):
             unicode(stdout)                     # Default
             unicode(stdout+stderr)              # with_stderr=True
             tuple(int(status), unicode(output)) # with_status=True
+
         """
 
         if GIT_PYTHON_TRACE and not GIT_PYTHON_TRACE == 'full':
@@ -98,7 +114,12 @@ class Git(object):
         else:
             stderr = None
 
+        if sys.platform == 'win32':
+            command = map(replace_carot, command)
+
         # Start the process
+        # Guard against thread-unsafe .git/index.lock files
+        cmdlock.acquire()
         while True:
             try:
                 proc = subprocess.Popen(command,
@@ -112,15 +133,23 @@ class Git(object):
                 # Some systems interrupt system calls and throw OSError
                 if e.errno == errno.EINTR:
                     continue
+                cmdlock.release()
                 raise e
-
         # Wait for the process to return
-        output = core.read_nointr(proc.stdout)
-        proc.stdout.close()
-        status = core.wait_nointr(proc)
+        try:
+            output = core.read_nointr(proc.stdout)
+            proc.stdout.close()
+            status = core.wait_nointr(proc)
+        except:
+            status = 202
+            output = str(e)
+
+        # Let the next thread in
+        cmdlock.release()
 
         if with_exceptions and status != 0:
-            raise errors.GitCommandError(command, status, output)
+            cmdstr = 'Error running: %s\n%s' % (' '.join(command), str(e))
+            raise errors.GitCommandError(cmdstr, status, output)
 
         if not with_raw_output:
             output = output.rstrip()
@@ -131,6 +160,10 @@ class Git(object):
             else:
                 print "%s -> %d" % (command, status)
 
+        if GIT_COLA_TRACE:
+            msg = 'trace: %s' % ' '.join(map(commands.mkarg, command))
+            cola.notifier().broadcast(signals.log_cmd, status, msg)
+
         # Allow access to the command's status code
         if with_status:
             return (status, output)
@@ -155,12 +188,12 @@ class Git(object):
                     args.append("--%s=%s" % (dashify(k), v))
         return args
 
-    def _call_process(self, method, *args, **kwargs):
+    def _call_process(self, cmd, *args, **kwargs):
         """
         Run the given git command with the specified arguments and return
         the result as a String
 
-        ``method``
+        ``cmd``
             is the command
 
         ``args``
@@ -176,6 +209,7 @@ class Git(object):
 
         Returns
             Same as execute()
+
         """
 
         # Handle optional arguments prior to calling transform_kwargs
@@ -190,52 +224,101 @@ class Git(object):
         ext_args = map(core.encode, args)
         args = opt_args + ext_args
 
-        call = ['git', dashify(method)]
+        call = ['git', dashify(cmd)]
         call.extend(args)
 
         return self.execute(call, **_kwargs)
 
 
-def shell_quote(*inputs):
+def replace_carot(cmd_arg):
     """
-    Quote strings so that they can be suitably martialled
-    off to the shell.  This method supports POSIX sh syntax.
-    This is crucial to properly handle command line arguments
-    with spaces, quotes, double-quotes, etc. on darwin/win32...
+    Guard against the windows command shell.
+
+    In the Windows shell, a carat character (^) may be used for
+    line continuation.  To guard against this, escape the carat
+    by using two of them.
+
+    http://technet.microsoft.com/en-us/library/cc723564.aspx
+
     """
+    return cmd_arg.replace('^', '^^')
 
-    regex = re.compile('[^\w!%+,\-./:@^]')
-    quote_regex = re.compile("((?:'\\''){2,})")
 
-    ret = []
-    for input in inputs:
-        if not input:
-            continue
+class GitCola(Git):
+    """
+    Subclass Git to provide search-for-git-dir
 
-        if '\x00' in input:
-            raise AssertionError,('No way to quote strings '
-                                  'containing null(\\000) bytes')
+    """
+    def __init__(self):
+        Git.__init__(self)
+        self.load_worktree(os.getcwd())
 
-        # = does need quoting else in command position it's a
-        # program-local environment setting
-        match = regex.search(input)
-        if match and '=' not in input:
-            # ' -> '\''
-            input = input.replace("'", "'\\''")
+    def load_worktree(self, path):
+        self._git_dir = path
+        self._worktree = None
+        self.worktree()
 
-            # make multiple ' in a row look simpler
-            # '\'''\'''\'' -> '"'''"'
-            quote_match = quote_regex.match(input)
-            if quote_match:
-                quotes = match.group(1)
-                input.replace(quotes, ("'" *(len(quotes)/4)) + "\"'")
+    def worktree(self):
+        if self._worktree:
+            return self._worktree
+        self.git_dir()
+        if self._git_dir:
+            curdir = self._git_dir
+        else:
+            curdir = os.getcwd()
 
-            input = "'%s'" % input
-            if input.startswith("''"):
-                input = input[2:]
+        if self._is_git_dir(os.path.join(curdir, '.git')):
+            return curdir
 
-            if input.endswith("''"):
-                input = input[:-2]
-        ret.append(input)
-    return ' '.join(ret)
+        # Handle bare repositories
+        if (len(os.path.basename(curdir)) > 4
+                and curdir.endswith('.git')):
+            return curdir
+        if 'GIT_WORK_TREE' in os.environ:
+            self._worktree = os.getenv('GIT_WORK_TREE')
+        if not self._worktree or not os.path.isdir(self._worktree):
+            if self._git_dir:
+                gitparent = os.path.join(os.path.abspath(self._git_dir), '..')
+                self._worktree = os.path.abspath(gitparent)
+                self.set_cwd(self._worktree)
+        return self._worktree
 
+    def is_valid(self):
+        return self._git_dir and self._is_git_dir(self._git_dir)
+
+    def git_path(self, *paths):
+        return os.path.join(self.git_dir(), *paths)
+
+    def git_dir(self):
+        if self.is_valid():
+            return self._git_dir
+        if 'GIT_DIR' in os.environ:
+            self._git_dir = os.getenv('GIT_DIR')
+        if self._git_dir:
+            curpath = os.path.abspath(self._git_dir)
+        else:
+            curpath = os.path.abspath(os.getcwd())
+        # Search for a .git directory
+        while curpath:
+            if self._is_git_dir(curpath):
+                self._git_dir = curpath
+                break
+            gitpath = os.path.join(curpath, '.git')
+            if self._is_git_dir(gitpath):
+                self._git_dir = gitpath
+                break
+            curpath, dummy = os.path.split(curpath)
+            if not dummy:
+                break
+        return self._git_dir
+
+    def _is_git_dir(self, d):
+        """From git's setup.c:is_git_directory()."""
+        if (os.path.isdir(d)
+                and os.path.isdir(os.path.join(d, 'objects'))
+                and os.path.isdir(os.path.join(d, 'refs'))):
+            headref = os.path.join(d, 'HEAD')
+            return (os.path.isfile(headref)
+                    or (os.path.islink(headref)
+                    and os.readlink(headref).startswith('refs')))
+        return False
